/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.lookandfeel;

import java.awt.Color;
import java.awt.color.ColorSpace;
import it.tidalwave.netbeans.lookandfeel.impl.LightDimmerController;

/*******************************************************************************
 *
 * A subclass of <code>Color</code> that can get dimmed out.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class DimmableColor extends Color // UIResource
  {
    private final LightDimmerController dimmer = LightDimmerController.getInstance();

    public DimmableColor (final int r, final int g, final int b)
      {
        super(r, g, b);
      }

    public DimmableColor (final int r, final int g, final int b, final int a)
      {
        super(r, g, b, a);
      }

    @Override
    public int getRed()
      {
        return Math.round(super.getRed() * dimmer.getIntensity());
      }

    @Override
    public int getGreen()
      {
        return Math.round(super.getGreen() * dimmer.getIntensity());
      }

    @Override
    public int getBlue()
      {
        return Math.round(super.getBlue() * dimmer.getIntensity());
      }
    
    // Don't override getAlpha()
    
    @Override
    public int getRGB()
      {
        final int rgb = super.getRGB();
        final int a = (rgb >>> 24) & 255;
        final int r = Math.round(((rgb >>> 16) & 255) * dimmer.getIntensity());
        final int g = Math.round(((rgb >>>  8) & 255) * dimmer.getIntensity());
        final int b = Math.round(((rgb >>>  0) & 255) * dimmer.getIntensity());
        
        return (a << 24) | (r << 16) | (g << 8) | b;
      }
    
    @Override
    public String toString()
      {
        return "DimmableColor[r:" + getRed() + ", g:" + getGreen() + ", b:" + getBlue() + "]";    
      }
    
    @Override
    public float[] getRGBComponents (final float[] values)
      {
        final float[] result = super.getRGBComponents(values);
        result[0] *= dimmer.getIntensity();
        result[1] *= dimmer.getIntensity();
        result[2] *= dimmer.getIntensity();
        
        if (values != null)
          {
            values[0] *= dimmer.getIntensity();
            values[1] *= dimmer.getIntensity();
            values[2] *= dimmer.getIntensity();
          }
        
        return result;
      }
    
    @Override
    public float[] getColorComponents (final ColorSpace colorSpace, final float[] values)
      {
        final float[] result = super.getColorComponents(colorSpace, values);
        result[0] *= dimmer.getIntensity();
        result[1] *= dimmer.getIntensity();
        result[2] *= dimmer.getIntensity();
        
        if (values != null)
          {
            values[0] *= dimmer.getIntensity();
            values[1] *= dimmer.getIntensity();
            values[2] *= dimmer.getIntensity();
          }
        
        return result;
      }
    
    @Override
    public float[] getComponents (final ColorSpace colorSpace, final float[] values)
      {
        final float[] result = super.getComponents(colorSpace, values);
        result[0] *= dimmer.getIntensity();
        result[1] *= dimmer.getIntensity();
        result[2] *= dimmer.getIntensity();
        
        if (values != null)
          {
            values[0] *= dimmer.getIntensity();
            values[1] *= dimmer.getIntensity();
            values[2] *= dimmer.getIntensity();
          }
        
        return result;
      }
    
    @Override
    public float[] getRGBColorComponents (float[] values)
      {
        final float[] result = super.getRGBColorComponents(values);
        result[0] *= dimmer.getIntensity();
        result[1] *= dimmer.getIntensity();
        result[2] *= dimmer.getIntensity();
        
        if (values != null)
          {
            values[0] *= dimmer.getIntensity();
            values[1] *= dimmer.getIntensity();
            values[2] *= dimmer.getIntensity();
          }
        
        return result;
      }
  }
